// Arduino_ROV_Controller
// 
// Written by Theron Wierenga, January 2016
//
// This program reads messages coming from Arduino_ROV_Joystick and sends commands to motor drivers for PWM
// It can read and send back battery data to Arduino_ROV_Joystick which can forward it to the PC
// 
// Defined commands sent to ROV
//
// LMF255 - left motor forward and PWM value
// LMR255 - left motor reverse and PWM value
// RMF255 - right motor forward and PWM value
// RMR255 - right motor reverse and PWM value
// UMU255 - up/down motor up and PWM value
// UMD255 - u/down motor down and PWM value
//
// Defined data sent back to joystick
// BAT followed by battery voltage as a float, end with \n
// CMP followed by 359, compass bearing going clockwise, end with \n
//
// Serial to ROV Arduino is D12, D13, RX, TX
//
// Left enable        D2, HIGH enables
// Left Forward PWM   D3
// Left Reverse PWM   D5
// Right enable       D4
// Right Forward PWM  D6
// Right Reverse PWM  D9
// U/D enable         D8
// U/D Forward (Down) PWM    D10
// U/D Reverse (Up)   PWM    D11
//
// SDA is A4
// SCL is A5
// Battery level is read on A0
// Optional pressure is read on A1
//
// These are for PWM

#define LMF   3
#define LMR   5
#define RMF   6
#define RMR   9
#define UMU   11
#define UMD   10

const int bat = A0;

#include <SoftwareSerial.h>
#include <Wire.h>

SoftwareSerial mySerial(12, 13); // RX, TX
String inputString = "", strTemp = "";
boolean stringComplete = false;

void setup() 
{
  pinMode(2, OUTPUT);
  pinMode(LMF, OUTPUT);
  pinMode(LMR, OUTPUT);
  pinMode(4, OUTPUT);
  pinMode(RMF, OUTPUT);
  pinMode(RMR, OUTPUT);
  pinMode(8, OUTPUT);
  pinMode(UMU, OUTPUT);
  pinMode(UMD, OUTPUT);
  
  // Turn Motors off
  analogWrite(LMF, 0);
  analogWrite(LMR, 0);
  analogWrite(RMF, 0);
  analogWrite(RMR, 0);
  analogWrite(UMU, 0);
  analogWrite(UMD, 0);

  // Enable motors
  digitalWrite(2, HIGH);
  digitalWrite(4, HIGH);
  digitalWrite(8, HIGH);

  Serial.begin(9600);
  // reserve 300 bytes for the inputString:
  inputString.reserve(300);
  mySerial.begin(9600);
}

void loop() 
{
    mySerialEvent();
    if (stringComplete)
    {
        Serial.print("inputString = ");
        Serial.println(inputString);
        String subStr = inputString.substring(0, 3);
        String temp = inputString.substring(3, 6);
        byte val = (byte)temp.toInt();
        if ((val >= 0) && (val <= 255))
        {
            Serial.print("****  ");
            Serial.print(subStr);
            Serial.print("  ");
            Serial.print(temp);
            Serial.print("  ");
            Serial.println(val);
            
            if (subStr == "LMF")
            {
              analogWrite(LMR, 0);
              analogWrite(LMF, val);
              
            }
            if (subStr == "LMR") 
            {
              analogWrite(LMF, 0);
              analogWrite(LMR, val);
            }
            
            if (subStr == "RMF") 
            {
              analogWrite(RMR, 0);
              analogWrite(RMF, val);
            }
            if (subStr == "RMR") 
            {
              analogWrite(RMF, 0);
              analogWrite(RMR, val);
            }
            
            if (subStr == "UMU") 
            {
              analogWrite(UMD, 0);
              analogWrite(UMU, val);
            }
            if (subStr == "UMD") 
            {
              analogWrite(UMU, 0);
              analogWrite(UMD, val);
            }
        }
        // Clear variables for next read
        stringComplete = false;
        inputString = "";
       
    }

    // Check 12 volt battery voltage
    battery();

    // Add a slight pause
    delay(200);
}

void mySerialEvent() 
{
    while ((mySerial.available() > 0) && (stringComplete == false))
    {
        // get the new byte:
        char inChar = (char)mySerial.read();
        // add it to the inputString:
        inputString += inChar;
        delay(2);                                                 // do i need this?
        // if the incoming character is a newline, set a flag
        // so the main loop can do something about it:
        if (inChar == '\n') 
        {
          stringComplete = true;
        }
    }
}

void battery()
{
     // Check 12 volt battery voltage
    float fbat = float(analogRead(bat));
    float voltage = ((fbat/1023.0)*5.0) * 2.734; // The 2.734 value was fudged to match a reading with a voltmeter
    Serial.print("Voltage = ");
    Serial.println(voltage);
    mySerial.print("BAT");
    mySerial.println(voltage);
}



